<?php if (!defined("IN_MYBB")) die("Access denied.");

if (!function_exists("threadpass_allowed_usergroups")) {
    function threadpass_allowed_usergroups($super_only=false) {
        global $mybb;

        if (!$super_only) {
            $allowed_usergroups = array_unique(array_merge(
                explode(",", $mybb->settings['threadpass_priv_supergroups']), 
                explode(",", $mybb->settings['threadpass_priv_usergroups'])
            ));
        } else {
            $allowed_usergroups = explode(",", $mybb->settings['threadpass_priv_supergroups']);
        }
        
        return $allowed_usergroups;
    }
}

if (!function_exists("threadpass_is_super")) {
    function threadpass_is_super($uid="") {
        global $mybb;
        
        $user = (!empty($uid)) ? get_user($uid) : $mybb->user;
        
        if (!$user) { return false; }
        $allowed_usergroups = threadpass_allowed_usergroups(true);
        
        $in_array = in_array($user['usergroup'], $allowed_usergroups);
        
        return (bool) $in_array;
    }
}

if (!function_exists("threadpass_allowed")) {
    function threadpass_allowed($thread="", $uid="") {
        global $mybb;
        
        $tid = $thread['tid'];
        
        $user = (empty($uid)) ? $mybb->user : get_user($uid);
        
        /* TODO:
         * (a) Check if password is stored in session:
         * -- Compare password with thread password. If correct:
         * -- -- Store authentication time in session.
         * 
         * (b) If thread has been edited since authentication time:
         * -- Repeat step a.
         */
        
        if (!empty($tid)) {
            // Check if this thread is password protected
            if ($thread['threadpass_protected'] != 0) {
                // Is this their thread, or are they a super user?
                if ($user['uid'] == $thread['uid'] || threadpass_is_super($user['uid'])) {
                    // You shall pass.
                    return true;
                }
                
                // Are they even priviliged?
                if (!in_array($user['usergroup'], threadpass_allowed_usergroups())) {
                    return false;
                }
                
                // And finally, if it's not their thread but they're allowed to use ThreadPass:
                // Have they authenticated with this thread?
                if (threadpass_is_authenticated($thread)) {
                    return true;
                }

                // Look user, I've given you enough chances to redeem yourself, but
                // regardless you still come here without any credentials or authorization
                // to back up your requests. For this reason, I'm denying you access to this thread
                // until you can bring credible versions of any of the items listed above.
                //
                // Sincerely, PHP.
                return false;
            } else {
                // Locked it isn't, this thread.
                // Allow access, I shall.
                return true;
            }
        }

        // We're not checking the allowance against a thread,
        // so just see if the user's group is allowed in general.
        $allowed_usergroups = threadpass_allowed_usergroups();
        
        return in_array($user['usergroup'], $allowed_usergroups);
    }
}

if (!function_exists("threadpass_is_authenticated")) {
    function threadpass_is_authenticated($thread, $uid="") {
        global $mybb;
        
        $user = (empty($uid)) ? $mybb->user : get_user($uid);
        
        if ($thread) {
            $tid = $thread['tid'];
            
            // Is this thread even protected?
            if ($thread['threadpass_protected'] != 0) {
                global $mybb;


                // Grab the authenticated data for this user, and unserialize it into an array
                $authentication = unserialize($user['threadpass_authenticated']);

				// NightFox: исправлено условие
                // If they've never authenticated with this thread...
                if (!isset($authentication[$tid]) || empty($authentication[$tid])) {
                    return false;
                }


                if (!empty($authentication[$tid])) {
                    // At this point, we can assume they've authenticated with this thread at least once.
                    // But has the thread's first post been edited since then?
                    $post = get_post($thread['firstpost']);
                
                    if ($authentication[$tid]['datetime'] < $post['edittime']) {
                        // It has.
                        // Re-authenticate them, or at least try to.
                        // This will pass them onto where they need to be if they
                        // pass authentication, or return false if they don't.
                        $passed = threadpass_authenticate($thread, true);

                        // If they failed to authenticate
                        if ($passed == False) {

                            // Clear their ID from the database, just to avoid this little step next time around.
                            threadpass_unauthenticate($thread);

                            return false;
                        }

                        // They've passed authentication, so return true
                        return true;
                    }
                } else {
                    // They have no credentials
                    return false;
                }
                
                // The thread hasn't been edited, so their credentials are still valid
                return true;
            }
            
            // This thread isn't even password-protected, WHAT ARE WE EVEN DOING HERE? THIS WHOLE SCRIPT IS OUT OF ORDER.
            return true;
        }
        
        // Thread is invalid, just return true and let the elves sort it out
        return true;
        
        /* THE TRUTH IS OUT THERE. 
               BECAUSE I RETURNED IT.
                   ALL OF IT. */
    }
}

if (!function_exists("threadpass_authenticate")) {
    function threadpass_authenticate($thread, $recheck=false) {
        global $mybb, $db;
        
        $tid = $thread['tid'];
        
        // Not going to check the thread's protection status, it's moot by this point.
        // Make sure they've not already authenticated with this thread.
        //if(!threadpass_is_authenticated($thread)) {
        
		// NightFox: новая логика
//      $check_password = ($recheck) ? $mybb->user['threadpass_password'] : md5($mybb->input['threadpass_password']);
        $check_password = ($recheck) ? $mybb->user['threadpass_password'] : md5(md5("a035e42c2d0b3b57017ae736545800fc").md5($mybb->input['threadpass_password']));

        // Compare the passwords
//      if (md5($thread['threadpass_password']) == $check_password) {
        if ($thread['threadpass_password'] == $check_password) {
            // Passed
            // Grab the authenticated data for this user, and unserialize it into an array
            $authenticated = unserialize($mybb->user['threadpass_authenticated']);
			
			//NightFox: исправил условие
            //$authenticated = (empty($authenticated)) ? $authenticated : array();
			If (empty($authenticated)) {
				$authenticated = array();
			}

            $authenticated[$tid] = array(
                'threadpass_password' => $check_password, // Store it in the database for later comparisons
                'datetime' => TIME_NOW
            );

            // Update the row in the the database
            $db->update_query("users", array("threadpass_authenticated" => serialize($authenticated)), "uid={$mybb->user['uid']}");
            
            // Updated the authenticated users for this thread
            $db->update_query("threads", array("threadpass_authenticated_users" => trim($thread['threadpass_authenticated_users'] . "|" . $mybb->user['uid'], "|")), "tid={$tid}");
            
            // Update the data in the variables for this session
            $mybb->user['threadpass_authenticated'] = serialize($authenticated);

            return true;
        } else {
            // Failed
            return false;
        }
        
        //} else {
        //    return true;
        //}
        
        // A call this part of the script Anette, because she's a catch-all.
        return false;
    }
}

if (!function_exists("threadpass_unauthenticate")) {
    function threadpass_unauthenticate($thread) {
        global $mybb, $db;
        
        if (!isset($mybb->user['threadpass_authenticated']) || empty($mybb->user['threadpass_authenticated'])) {
            return true;
        }
        
        $tid = $thread['tid'];
        
        // Grab the authenticated data for this user, and unserialize it into an array
        $authenticated = unserialize($mybb->user['threadpass_authenticated']);
        
        // Is this user authenticated in this thread?
        if (isset($authenticated[$tid])) {
            // Clear the thread from the users authentication set
            unset($authenticated[$tid]);
            
            // Update the row in the the database
            $db->update_query("users", array("threadpass_authenticated" => serialize($authenticated)), "uid={$mybb->user['uid']}");
            
            // Update the data in the $mybb variable
            $mybb->user['threadpass_authenticated'] = serialize($authenticated);
        } else {
            // Else if it doesn't exist, just return true since they're already unauthenticated
            return true;
        }
    }
}

if (!function_exists("threadpass_error")) {
    function threadpass_error($error="", $threadpass_return="", $title="") {
	global $header, $footer, $theme, $headerinclude, $db, $templates, $lang, $mybb, $plugins, $threadpass_form, $threadpass_return, $thread;

	$error = $plugins->run_hooks("error", $error);
	if(!$error)
	{
		$error = $lang->unknown_error;
	}

	// AJAX error message?
	if($mybb->input['ajax'])
	{
		// Send our headers.
		@header("Content-type: text/html; charset={$lang->settings['charset']}");
		echo "<error>{$error}</error>\n";
		exit;
	}

	if(!$title)
	{
		$title = $mybb->settings['bbname'];
	}

	$timenow = my_date($mybb->settings['dateformat'], TIME_NOW) . " " . my_date($mybb->settings['timeformat'], TIME_NOW);
	reset_breadcrumb();
	add_breadcrumb($lang->error);
        
        if (in_array($mybb->user['usergroup'], threadpass_allowed_usergroups())) {
            eval("\$threadpass_form = \"".$templates->get('threadpass_form')."\";");
        }
        
        //pr($mybb->user, false);
        //pr($thread, false);

	eval("\$errorpage = \"".$templates->get("error")."\";");
	output_page($errorpage);

	exit;
    }
}

if (!function_exists("threadpass_update_subs")) {
    function threadpass_update_subs($tid, $uid="") {
        global $db, $cache;
        
        $user = (empty($uid)) ? $mybb->user : get_user($uid);
        $subs = $db->simple_select("threadsubscriptions", "*", "tid={$tid}");
        
        // Loop over all of the subscribed threads and remove 
        // any from users that aren't allowed to view this thread.
        while($sub = $db->fetch_array($subs)) {
            if (!threadpass_allowed(get_thread($tid), $sub['uid'])) {
                $db->delete_query("threadsubscriptions", "sid={$sub['sid']}");
            }
        }
    }
}

/* End: /inc/plugins/Beardy/threadpass/functions.php */