<?php if (!defined("IN_MYBB")) die("Access Denied");

/*
 * Notes:
 * Fix attachments in RSS feed and Portal (see syndication.php Ln 169)
 */

function threadpass_info() {
    global $db, $mybb;
    
    $settings_link = get_settings_link("threadpass");
    $prefix = get_plugin_prefix();

    return array(
        "name"		=> $prefix."ThreadPass",
        "description"	=> $settings_link."Allow threads to be password protected.",
        "website"	=> "",
        "author"	=> "Beardy",
        "authorsite"	=> "",
        "version"	=> "1.0.3d", 
        'guid'        	=> ''
    );
}

function threadpass_is_installed() {
    global $db;
		
    return (bool) $db->fetch_field($db->simple_select("settinggroups", "*", "name='threadpass'"), "gid");
}

function threadpass_install() {
    global $db;
    
    acp_settings_add(
        /* Setting group info */
        array(
            'name' => 'threadpass',
            'title' => 'ThreadPass Settings',
            'description' => 'Settings to configure the ThreadPass plugin.',
            'isdefault' => 0,
        ),
        
        /* Settings */
        array(
            array(
                'name' => 'threadpass_priv_usergroups',
                'title' => 'Privileged Usergroups',
                'description' => "Select all of the groups that will be allowed to password protect their. These usergroups are also the only groups allowed to login to password-protected threads.",
                'optionscode' => form_groupselect_box(),
                'value' => '2,3,4,6'
            ),
            
            array(
                'name' => 'threadpass_priv_supergroups',
                'title' => 'Authority Groups',
                'description' => "Select all of the groups that will have control over any password-protected thread.",
                'optionscode' => form_groupselect_box(),
                'value' => '3,4,6'
            ),
            
            array(
                'name' => 'threadpass_thread_affix_icon',
                'title' => 'Thread Affix Icon',
                'description' => "Choose the icon that will be affixed to protected threads.",
                'optionscode' => 'text',
                'value' => 'images/threadpass_affix.png'
            )
        )
    );
    
    $db->add_column("threads", "threadpass_password", "varchar(150)");
    $db->add_column("threads", "threadpass_protected", "int(1) DEFAULT '0'");
    $db->add_column("threads", "threadpass_authenticated_users", "text");
    
    $db->add_column("users", "threadpass_authenticated", "text");
}

function threadpass_uninstall() {
    global $db;
    
    // Remove the setting group and settings
    acp_settings_remove("threadpass");
    
    // Remove the columns edits
    if ($db->field_exists("threadpass_password", "threads")) $db->drop_column("threads", "threadpass_password");
    if ($db->field_exists("threadpass_protected", "threads")) $db->drop_column("threads", "threadpass_protected");
    if ($db->field_exists("threadpass_authenticated_users", "threads")) $db->drop_column("threads", "threadpass_authenticated_users");
    
    if ($db->field_exists("threadpass_authenticated", "users")) $db->drop_column("users", "threadpass_authenticated");
}

function threadpass_activate() {
    global $db;
    
    $templates = array(
        array(
            "title"     => "threadpass_row",
            "template"  => $db->escape_string('<td class="trow2" width="20%"><strong>{$lang->password}</strong></td><td class="trow2"><input type="password" class="textbox" name="threadpass_password" value="{$threadpass_password_value}" size="60" maxlength="150" tabindex="1" /></td></tr>'),
            "sid"       => "-1",
            "version"	=> "1.0",
            "dateline"	=> time(),
        ),
        
        array(
            "title"     => "threadpass_thread_affix",
            "template"  => $db->escape_string('<img title="{$lang->passprotect_title}" src="{$affix_icon}">'),
            "sid"       => "-1",
            "version"	=> "1.0",
            "dateline"	=> time(),
        ),
        
        array(
            "title"     => "threadpass_form",
            "template"  => '
<br /><br />
<form action="misc.php?action=threadpass_authenticate" method="POST">
  <input type="hidden" name="r" value="{$threadpass_return}" />
  <input type="hidden" name="my_post_key" value="{$mybb->post_code}" />
  <input type="hidden" name="tid" value="{$mybb->input[\\\'tid\\\']}" />
  {$lang->password}: <input type="password" name="threadpass_password" /> <input type="submit" value="{$lang->auth}" />
</form>',
            "sid"       => "-1",
            "version"	=> "1.0",
            "dateline"	=> time(),
        ),
        
        array(
            "title"     => "threadpass_authed_popup",
            "template"  => $db->escape_string('
<a style="position: relative; float: left; font-size: 11px; top: 10px; cursor: pointer;" href="#" onclick="javascript:window.open(\'misc.php?action=threadpass_who_authed&tid={$thread[\'tid\']}\',\'\',\'width=300,height=400\')">{$lang->showauthusers}</a>'),
            "sid"       => "-1",
            "version"	=> "1.0",
            "dateline"	=> time(),
        ),
        
        array(
            "title"     => "threadpass_who_authed",
            "template"  => '
<html>
<head>
<title>{$lang->whoauth}</title>
{$headerinclude}
</head>
<body style="margin:0px;top:0px;left:0px" class="trow2">
<table width="100%" cellspacing="{$theme[\\\'borderwidth\\\']}" cellpadding="{$theme[\\\'tablespace\\\']}" border="0" align="center" class="tborder">
<tr>
<td colspan="2" class="thead"><strong>{$lang->authusers} ({$authed_num})</strong></td>
</tr>
<tr>
<td class="tcat"><span class="smalltext" style="text-align: left"><strong><a>{$lang->username}</a></strong></span></td>
</tr>
{$whoauthed}
<tr>
<td colspan="2" class="thead" align="center"><span class="smalltext">[<a href="javascript:self.close();">{$lang->close}</a>]</span></td>
</tr>
</table>
</body>
</html>',                
            "sid"       => "-1",
            "version"	=> "1.0",
            "dateline"	=> time(),
        ),
        
        array(
            "title"     => "threadpass_who_authed_user",
            "template"  => '
<tr>
<td class="trow1">{$profile_link}</a></td>
</tr>',                
            "sid"       => "-1",
            "version"	=> "1.0",
            "dateline"	=> time(),
        )
        
    );
    
    foreach ($templates as $template) {
        $db->insert_query("templates", $template);
    }
    
    require_once MYBB_ROOT."/inc/adminfunctions_templates.php";
    find_replace_templatesets('newthread', "#".preg_quote('{$posticons}')."#i", '{$threadpass_row}{$posticons}');
    find_replace_templatesets('editpost', "#".preg_quote('{$posticons}')."#i", '{$threadpass_row}{$posticons}');
    
    find_replace_templatesets('forumdisplay_thread', "#".preg_quote("{\$thread['multipage']}")."#i", "{\$thread['multipage']}{\$threadpass_affix}");
    find_replace_templatesets('error', "#".preg_quote('{$error}')."#i", "{\$error}{\$threadpass_form}");
    
    find_replace_templatesets('showthread', "#".preg_quote('{$ratethread}')."#i", '{$threadpass_authed_popup}{$ratethread}');
}

function threadpass_deactivate() {
    global $db;
    
    // Remove the templates
    $db->delete_query("templates", "title LIKE 'threadpass%'"); 
    
    // Remove the template edits
    require_once MYBB_ROOT."/inc/adminfunctions_templates.php";
    find_replace_templatesets('newthread', "#".preg_quote('{$threadpass_row}')."#i", '');
    find_replace_templatesets('editpost', "#".preg_quote('{$threadpass_row}')."#i", '');
    
    find_replace_templatesets('forumdisplay_thread', "#".preg_quote("{\$threadpass_affix}")."#i", '');
    find_replace_templatesets('error', "#".preg_quote("{\$threadpass_form}")."#i", '');
    
     find_replace_templatesets('showthread', "#".preg_quote('{$threadpass_authed_popup}')."#i", '');
}

/* End: /inc/Beardy/threadpass/mybb_functions.php */